"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.upgradeHTML = upgradeHTML;
exports.setRoot = setRoot;
exports.boot = boot;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _nodeToolkit = require("@tech_query/node-toolkit");

var _jsdom = require("jsdom");

var _library = require("./library");

var _path = require("path");

var _fsExtra = require("fs-extra");

var _command = require("create-es-pack/dist/command");

function equalLibrary(element, type, key, name, file) {
  const URI = element[key];
  return element.tagName.toLowerCase() === type && URI.includes(name) && URI.includes(file);
}

const filter = [].filter,
      this_module = (0, _nodeToolkit.currentModulePath)(),
      preTag = ['META', 'TITLE', 'BASE', 'LINK'];
/**
 * @param {String} code - HTML source
 *
 * @return {JSDOM}
 */

function upgradeHTML(code) {
  const page = new _jsdom.JSDOM(code);
  const document = page.window.document;
  const list = Array.from(document.querySelectorAll(Object.keys(_library.tagAttribute))),
        point = filter.call(document.head.childNodes, node => preTag.includes(node.tagName) || node.nodeType === 8).slice(-1)[0];

  const script = _library.library.map(({
    type,
    name,
    file,
    path
  }) => {
    file = file || name;
    var _tagAttribute$type = _library.tagAttribute[type],
        key = _tagAttribute$type.key,
        kind = _tagAttribute$type.kind,
        element;

    if (!(element = list.find(item => equalLibrary(item, type, key, name, file)))) {
      element = document.createElement(type);
      element[key] = `node_modules/${name}/${path || ''}${file}${/\.\w+$/.test(file) ? '' : `.min.${kind}`}`;
      if (type === 'link') element.rel = 'stylesheet';
    }

    return element;
  });

  if (point) point.after(...script);else document.head.append(...script);
  return page;
}

function setRoot(_x) {
  return _setRoot.apply(this, arguments);
}
/**
 * Boot a directory as a WebCell project
 *
 * @param {String}     cwd      - Project path
 * @param {String|URL} [remote] - URL of Git repository
 * @param {?Boolean}   app      - Add extensions for WebSite or WebApp
 */


function _setRoot() {
  _setRoot = (0, _asyncToGenerator2.default)(function* (cwd) {
    ['package', '.eslintrc'].forEach(name => {
      var file = (0, _path.join)(cwd, name + '.json');
      (0, _fsExtra.writeJSONSync)(file, Object.assign((0, _fsExtra.readJSONSync)(file), (0, _fsExtra.readJSONSync)((0, _path.join)(this_module, `../../template/${name}.json`))));
    });
    yield (0, _fsExtra.copy)((0, _path.join)(this_module, '../../template/'), cwd, {
      overwrite: false
    });
    const file = (0, _path.join)(cwd, 'index.html');
    yield (0, _fsExtra.outputFile)(file, upgradeHTML((yield (0, _fsExtra.readFile)(file))).serialize());
  });
  return _setRoot.apply(this, arguments);
}

function boot() {
  return _boot.apply(this, arguments);
}

function _boot() {
  _boot = (0, _asyncToGenerator2.default)(function* (cwd = '.', remote, app) {
    console.time('Boot project');
    yield (0, _command.boot)(cwd, remote);
    yield (0, _nodeToolkit.step)('Enhance root files', setRoot.bind(null, cwd));
    const child_option = {
      stdio: 'inherit',
      cwd
    };
    yield (0, _nodeToolkit.step)('WebCell dependency',
    /*#__PURE__*/
    (0, _asyncToGenerator2.default)(function* () {
      (0, _fsExtra.removeSync)((0, _path.join)(cwd, 'package-lock.json'));
      yield (0, _nodeToolkit.spawn)('npm', ['install'], child_option);
      if (app) yield (0, _nodeToolkit.spawn)('npm', ['install', 'cell-router', 'data-scheme'], child_option);
      yield (0, _nodeToolkit.ensureCommand)('web-cell');
    }));
    yield (0, _nodeToolkit.step)('Git commit',
    /*#__PURE__*/
    (0, _asyncToGenerator2.default)(function* () {
      yield (0, _nodeToolkit.spawn)('web-cell', ['new', 'cell-hello', 'name,value'], child_option);
      yield (0, _nodeToolkit.spawn)('git', ['add', '.'], child_option);
      yield (0, _nodeToolkit.spawn)('git', ['commit', '-m', '[Add] WebCell framework'], child_option);
    }));
    console.info('--------------------');
    console.timeEnd('Boot project');
    console.info('');
  });
  return _boot.apply(this, arguments);
}
//# sourceMappingURL=core.js.map